︠13719ce7-f8be-4791-9462-0548c4afd8b3︠
2+2
︡4aeb5d27-cfff-4209-9c79-50412d4cd63a︡︡
︠0f6c9e1e-2397-4f02-8f65-983026341913︠
def rank_zero_twists(A, n):
    """
    INPUT:
        - A -- rank > 0 modular abelian variety  
        - n -- bound for search
    OUTPUT:
        - list of integers d with |d| <= n such that twisting A by the character (d/-)
          results in an abelian variety with analytic rank 0.
    """
         
    #from sage.modular.abvar.abvar import is_ModularAbelianVariety
    #if is_ModularAbelianVariety(A):
    #    A = A.modular_symbols(sign=0)
    if A.sign() != 0:
        raise ValueError, "A must have sign 0"
    phi = A.rational_period_mapping()    
    N = A.level()
    M = A.ambient_module()
    if phi(M([0,oo])) != 0:
        raise ValueError, "A must have analytic rank > 0"
    v = []
    for d in [-n..n]:
        if gcd(A.level(), d) == 1:
            eps = kronecker_character(d)
            if eps.is_primitive() and eps.conductor() > 1 : #and eps(-N) == 1:
                #print "trying %s"%d
                w = M.twisted_winding_element(0, eps)
                if phi(w) != 0:
                     #print d
                     v.append(d)
    return v
︡0406147e-52da-4afb-bb7a-4e7f01b2f1ba︡︡
︠42ca8ed3-0400-42f9-99b4-6d32d346c5ce︠
def higher_rank_twists(A, n):
    """
    INPUT:
        - A -- rank 0 modular abelian variety or sign 0 modular symbols space 
        - n -- bound for search
    OUTPUT:
        - list of integers d with |d| <= n such that twisting A by the character (d/-)
          results in an abelian variety with *even* positive analytic rank.
    """
         
    from sage.modular.abvar.abvar import is_ModularAbelianVariety
    if is_ModularAbelianVariety(A):
        A = A.modular_symbols(sign=0)
    if A.sign() != 0:
        raise ValueError, "A must have sign 0"
    phi = A.rational_period_mapping()    
    N = A.level()
    M = A.ambient_module()
    if phi(M([0,oo])) == 0:
        raise ValueError, "A must have analytic rank 0"
    v = []
    for d in [-n..n]:
        if gcd(A.level(), d) == 1:
            eps = kronecker_character(d)
            if eps.is_primitive() and eps.conductor() > 1 and eps(-N) == 1:
                #print "trying %s"%d
                w = M.twisted_winding_element(0, eps)
                if phi(w) == 0:
             
                     v.append(d)
    return v
︡fe513187-57bb-4a9e-b97f-1935a804a7d8︡︡
︠65ad7b2e-7373-40c4-9300-546d4ffb0c5bi︠
%html
<p>Here we find rank 0 twists of a rank 2 elliptic curve:</p>

︡587dfd65-82ad-472c-bbfd-69f0ed65adf2︡{"html": "<p>Here we find rank 0 twists of a rank 2 elliptic curve:</p>"}︡
︠dcefb77a-9034-4856-816d-06302813cc89︠
M = ModularSymbols(389,2).cuspidal_submodule().new_subspace().decomposition()[0]
L = rank_zero_twists(M,50); L
︡db96f75f-1010-4f9e-ae50-1ca40985d791︡{"stdout": "[-50, -48, -43, -40, -39, -38, -37, -34, -33, -32, -31, -29, -27, -26, -23, -22, -21, -18, -15, -14, -12, -10, -8, -3, -2, 5, 6, 7, 11, 13, 17, 19, 20, 24, 28, 30, 35, 41, 42, 44, 45]"}︡
︠97ac24a5-953e-469c-8eed-afdced8706ba︠
E = EllipticCurve('389a')
for l in L:
    print l, E.quadratic_twist(l).analytic_rank()
︡dbd2f1f3-a529-4898-9f51-a47c27ec82bd︡{"stdout": "-50 0\n-48 0\n-43 0\n-40 0\n-39 0\n-38 0\n-37 0\n-34 0\n-33 0\n-32 0\n-31 0\n-29 0\n-27 0\n-26 0\n-23 0\n-22 0\n-21 0\n-18 0\n-15 0\n-14 0\n-12 0\n-10 0\n-8 0\n-3 0\n-2 0\n5 0\n6 0\n7 0\n11 0\n13 0\n17 0\n19 0\n20 0\n24 0\n28 0\n30 0\n35 0\n41 0\n42 0\n44 0\n45 0"}︡
︠cfb9cb09-ccba-456a-9252-949c780accedi︠
%html
<p>But for higher dimension, how large does $n$ need to be ?!</p>

︡a2a028f9-1f92-4e59-9b05-ccb9201c7df4︡{"html": "<p>But for higher dimension, how large does $n$ need to be ?!</p>"}︡
︠39fa8a7b-7937-4b59-bbf8-2b5aab06fb23︠
M = ModularSymbols(93,2).cuspidal_submodule().new_subspace().decomposition()[0]; M
L = rank_zero_twists(M,100); L
︡ffc4ca9f-5a6c-435f-ba75-4994c310ead3︡{"stdout": "[-100, -97, -94, -92, -89, -86, -83, -82, -77, -76, -74, -70, -68, -67, -65, -64, -53, -49, -44, -40, -29, -28, -26, -25, -23, -19, -17, -16, -11, -10, -7, -4, -1, 2, 5, 8, 13, 14, 20, 22, 32, 34, 35, 37, 38, 41, 43, 46, 47, 50, 52, 55, 56, 58, 59, 61, 71, 73, 79, 80, 85, 88, 91, 95, 98]"}︡
︠8d84cae0-258f-4bae-9574-5a2182189cb7︠
# sage: from psage.modform.rational.padic_lseries import *
       # sage: D = J0(93).decomposition()
       # sage: A = D[0]
      #  sage: L = pAdicLseriesOrdinary(A, 7)
       # sage: lval = L.series(n=2, 37)[0] #also, 5, 56
︡f0d01f11-6681-46d4-b6c6-731458378d46︡︡
︠3ff2a3b7-2553-43c0-b396-6857285cf438︠
#D = J0(188).decomposition()
#        sage: A = D[-2]
D = ModularSymbols(188,2).cuspidal_submodule().new_subspace().decomposition()
A = D[0]
#        sage: L = pAdicLseriesOrdinary(A, 7)
︡be715f12-dfae-4b88-bd7c-969f150c34fa︡︡
︠5cf46253-1756-4405-975f-eebdd94a1157︠
p = 11
A = []
chi = kronecker_character(p)
for l in L:
    if chi(l) == 1 and fundamental_discriminant(l) == l:
        A.append(l)
print A
︡16b35ccf-5837-401a-9cbb-dd176861bd9a︡︡
︠67ab6879-e072-4612-85bd-2a90210144ea︠
[-87, -43, -39, -35, -19, 5, 69, 93]
︡59be9dc1-86e7-45e7-ae0a-dd27ab33d8df︡︡
︠a5ac88b2-6bee-4ed0-8a27-cb68e8bde434︠
M = ModularSymbols(389,2).cuspidal_submodule().new_subspace().decomposition()[0]; M
L = rank_zero_twists(M,100); L
︡f9732764-8d5b-458e-8267-96f56ed66fd0︡{"stdout": "[-98, -92, -90, -89, -88, -84, -83, -82, -75, -72, -71, -70, -61, -60, -57, -56, -53, -51, -50, -48, -43, -40, -39, -38, -37, -34, -33, -32, -31, -29, -27, -26, -23, -22, -21, -18, -15, -14, -12, -10, -8, -3, -2, 5, 6, 7, 11, 13, 17, 19, 20, 24, 28, 30, 35, 41, 42, 44, 45, 52, 54, 55, 58, 59, 62, 63, 66, 68, 69, 73, 74, 76, 77, 78, 79, 80, 85, 87, 96, 97, 99]"}︡
︠75eff6e5-ab3c-4a30-85a1-dca1fb0d2e74︠
M= ModularSymbols(167,2).cuspidal_submodule().new_subspace().decomposition()[0]
L167 = rank_zero_twists(M,100); L167
︡f48297ce-2fbb-4f40-9c28-5d3c38e30b28︡{"stdout": "[-92, -91, -90, -86, -83, -82, -80, -79, -78, -74, -73, -71, -70, -69, -68, -67, -60, -52, -51, -46, -45, -43, -41, -40, -39, -37, -35, -34, -30, -26, -23, -20, -17, -15, -13, -10, -5, 5, 10, 13, 15, 17, 20, 23, 26, 30, 34, 35, 37, 39, 40, 41, 43, 45, 46, 51, 52, 55, 59, 60, 67, 68, 69, 70, 71, 73, 74, 78, 79, 80, 82, 83, 86, 90, 91, 92, 95]"}︡
︠0c197234-a2ad-422b-b8f9-3cd2ac4857ab︠
p = 19
A = []
chi = kronecker_character(p)
for l in L167:
    if chi(l) == 1 and fundamental_discriminant(l) == l:
        A.append(l)
print A
︡33f9295b-9b04-45be-ac16-e1fe3edcb9ae︡{"stdout": "[-91, -79, -71, -67, -51, -15, 5, 17, 73]"}︡
︠0a338d4a-c965-4815-86dd-964ee5853b8e︠
from psage.modform.rational.padic_lseries import *
D = J0(147).decomposition()[-2]
L = pAdicLseriesOrdinary(D, 19)
lval = L.series(2, 5)[0]
alpha1,alpha2 = L.alpha()
eps = (1-1/alpha1)^2*(1-1/alpha2)^2
-lval/eps
︡6746924a-3c46-44ea-bbbf-d864fe8747aa︡︡
︠5349557e-4391-451b-9a15-4a35fe23a61c︠
M = ModularSymbols(133,2).cuspidal_submodule().new_subspace().decomposition()[0]
L133 = rank_zero_twists(M,100); L133
︡655a2cd1-b2da-4f01-b31d-e3751cfc2770︡{"stdout": "[-100, -99, -97, -94, -93, -92, -90, -89, -85, -81, -75, -74, -69, -64, -59, -58, -52, -48, -44, -43, -41, -40, -39, -36, -34, -33, -31, -30, -27, -25, -23, -16, -13, -12, -11, -10, -9, -4, -3, -1, 2, 5, 6, 8, 15, 17, 18, 20, 22, 24, 26, 29, 32, 37, 45, 46, 47, 50, 51, 53, 54, 55, 60, 61, 62, 65, 66, 67, 68, 71, 72, 73, 78, 79, 80, 82, 83, 87, 88, 96]"}︡
︠80ec05f7-505e-41c5-b241-6f5dbb1f1165︠
p = 29
A = []
chi = kronecker_character(p)
for l in L133:
    if chi(l) == 1 and fundamental_discriminant(l) == l:
        A.append(l)
print A
︡7f4c2e94-e982-435e-897a-434b05d5c683︡{"stdout": "[-59, -52, -23, -4, 5, 24, 53, 65, 88]"}︡
︠c1cdd78c-a451-4f79-b553-846dba099840︠
from psage.modform.rational.padic_lseries import *
D = J0(133).decomposition()[2]
L = pAdicLseriesOrdinary(D, 29)
lval = L.series(2, 5)[0]
alpha1,alpha2 = L.alpha()
eps = (1-1/alpha1)^2*(1-1/alpha2)^2
-lval/eps
︡f56cbc05-ef10-482f-9c3a-dcc23030d3b5︡︡
︠f564db92-d643-4d54-92fa-8c198a39b7f6︠
D = ModularSymbols(147,2).cuspidal_submodule().new_subspace().decomposition()
A = D[-2]
phi = A.rational_period_mapping()    
M = A.ambient_module()
phi(M([0,oo]))
︡5d350ee0-00d4-4526-a70a-00ca53a3e4fc︡{"stdout": "(0, 0, 0, 0)"}︡
︠589d9abc-91ff-4c3d-9e66-7c64c3fcae99︠
M= ModularSymbols(147,2).cuspidal_submodule().new_subspace().decomposition()[-2]
L147 = rank_zero_twists(M,100); L147
︡14a1693a-9b07-4791-a1f8-0d6ab24eb327︡{"stdout": "[-92, -89, -83, -80, -74, -71, -68, -65, -62, -59, -53, -50, -47, -44, -41, -32, -29, -26, -23, -20, -17, -11, -8, -5, -2, 2, 5, 8, 11, 17, 20, 23, 26, 29, 32, 38, 41, 44, 47, 50, 53, 59, 62, 65, 68, 71, 74, 80, 83, 89, 92, 95]"}︡
︠0f4f7fe2-c22a-47ca-9e39-edfef19c2065︠
p = 31
A = []
chi = kronecker_character(p)
for l in L147:
    if chi(l) == 1 and fundamental_discriminant(l) == l:
        A.append(l)
print A
︡cf3ee7a2-90db-42a9-89e8-041000a8022e︡{"stdout": "[-83, -23, -11, 5, 41]"}︡
︠377bdcdb-b491-4ad2-b64a-ad6a676b9f46︠
from psage.modform.rational.padic_lseries import *
D = J0(147).decomposition()[-2]
L = pAdicLseriesOrdinary(D, 31)
lval = L.series(2, 41)[0]
alpha1,alpha2 = L.alpha()
eps = (1-1/alpha1)^2*(1-1/alpha2)^2    #(5,16), (41,16)
-lval/eps
︡76184993-d2d2-4f22-9004-7cc0f166e66c︡︡
︠65eb7dd7-0bbb-4a37-b895-a480472bc210i︠
%html
<p>Higher rank twists: double check with an elliptic curve:</p>

︡b7fd8d8e-5e31-47e6-84c2-60811e78f943︡{"html": "<p>Higher rank twists: double check with an elliptic curve:</p>"}︡
︠ec0c83dc-d237-4e89-969b-777e0018719a︠
higher_rank_twists(ModularSymbols(11,2).cuspidal_subspace(), 75)
︡7eb89ad5-ccd7-4bad-b554-273491e7251f︡{"stdout": "[-47, 58]"}︡
︠3c74ad15-22fb-4ff7-8d8a-6a52008c7e1c︠
EllipticCurve('11a').quadratic_twist(-47).analytic_rank()
︡1147f44b-c871-4d54-8499-335ebe5d99b7︡{"stdout": "2"}︡
︠744e7c99-488a-428a-a631-27f62d92136bi︠
%html
<p>Now the interesting search</p>

︡4aa0af17-f55f-419b-a54e-7594b66a66f8︡{"html": "<p>Now the interesting search</p>"}︡
︠b7df325a-6ba1-4622-8631-f439492aa20e︠
higher_rank_twists(ModularSymbols(23,2).cuspidal_subspace(), 150)
︡66fe54b0-f26b-4ac3-850d-7048f09bcb15︡{"stdout": "[59]"}︡
︠2b15716f-1dc4-4ba8-ac5a-7c19f43a0520︠
kronecker_character(59).conductor()
︡e110cb34-9c09-4266-badc-7e07fd215d20︡{"stdout": "236"}︡
︠9a7427d3-2025-4f61-9125-29ec6f0d7748︠
higher_rank_twists(ModularSymbols(31,2).cuspidal_subspace(), 150)
︡970d619e-3225-4387-88f8-919d394b512c︡{"stdout": "[-143, -131, -47]"}︡
︠b8bf4a82-dce2-4554-9e66-0a562acf1578︠
kronecker_character(-47).conductor()
︡cb2f9a8c-af60-4cfa-b2d3-c959acad4508︡{"stdout": "47"}︡
︠db1fb152-703f-4334-8b43-b52e56116341︠
A=ModularSymbols(35,2).cuspidal_subspace().new_subspace().decomposition()[1]; print A
︡2d27f99f-df70-4bf3-9084-fa2cabf2e0ca︡{"stdout": "Modular Symbols subspace of dimension 4 of Modular Symbols space of dimension 9 for Gamma_0(35) of weight 2 with sign 0 over Rational Field"}︡
︠8361177e-109b-4690-a58b-2f1a33af538c︠
higher_rank_twists(A, 75)
︡1dc37f50-df4a-4f55-8886-a72e06d73ff8︡{"stdout": "[]"}︡
︠221e2b62-03be-4ccd-8ee2-a6847db7594e︠
A=ModularSymbols(39,2).cuspidal_subspace().new_subspace().decomposition()[1]; print A
︡7db75b66-0b2b-4d4c-9f13-2f23ad23ecd2︡{"stdout": "Modular Symbols subspace of dimension 4 of Modular Symbols space of dimension 9 for Gamma_0(39) of weight 2 with sign 0 over Rational Field"}︡
︠fbd37560-96b5-4225-a42a-d503f2eebbfe︠
higher_rank_twists(A, 75)
︡3b0b3558-a37b-4cba-9647-af3e8b8cd40c︡{"stdout": "[]"}︡
︠753e2191-eb23-4cab-bc54-79b884767a16︠
A=ModularSymbols(63,2).cuspidal_subspace().new_subspace().decomposition()[1]; print A
︡d8837dc3-0a52-45d3-addc-c4a15d2c4214︡{"stdout": "Modular Symbols subspace of dimension 4 of Modular Symbols space of dimension 17 for Gamma_0(63) of weight 2 with sign 0 over Rational Field"}︡
︠9b40246e-e05d-4e4f-90db-c6b1b4bb73c3︠
higher_rank_twists(A, 75)
︡0033db07-81fc-4b03-9cd9-15c326bfd80b︡{"stdout": "[]"}︡
︠b366c47a-8c1e-4748-b420-2390c983a222︠
A=ModularSymbols(65,2).cuspidal_subspace().new_subspace().decomposition()[1]; print A
︡25ae67b6-5705-4ffc-bf95-7522f314a482︡{"stdout": "Modular Symbols subspace of dimension 4 of Modular Symbols space of dimension 13 for Gamma_0(65) of weight 2 with sign 0 over Rational Field"}︡
︠9fc8a205-1046-48da-8e20-fceabb4ba4ef︠
higher_rank_twists(A, 75)
︡411e9b11-9380-49a3-878d-492f5574864c︡{"stdout": "[]"}︡
︠46db810b-6793-49d5-9584-7b6b5d294176︠
A=ModularSymbols(65,2).cuspidal_subspace().new_subspace().decomposition()[2]; print A
︡42d6e420-7754-4571-b01f-1cc6b27365d5︡{"stdout": "Modular Symbols subspace of dimension 4 of Modular Symbols space of dimension 13 for Gamma_0(65) of weight 2 with sign 0 over Rational Field"}︡
︠38755144-17b0-48af-a1ba-fead9b8130e0︠
higher_rank_twists(A, 75)
︡7b9ee073-e07a-4b13-bb25-037b5a530feb︡{"stdout": "[]"}︡
︠7847e29f-112b-4574-91cb-aca2ebed0c09︠
A=ModularSymbols(87,2).cuspidal_subspace().new_subspace().decomposition()[0]; print A
︡6d89f6b9-e154-45a9-9f40-6c34038d0213︡{"stdout": "Modular Symbols subspace of dimension 4 of Modular Symbols space of dimension 21 for Gamma_0(87) of weight 2 with sign 0 over Rational Field"}︡
︠6246db68-fd1f-45f8-8461-7b763c74e116︠
higher_rank_twists(A, 75)
︡6d6e28ae-4163-4b01-aeac-be817c36280b︡{"stdout": "[]"}︡
